#!/usr/bin/env python
# -*- coding: utf-8 -*-

# RSUV3 Python Library.

"""
RSUV3 Python Library.
~~~~

:author: Greg Albrecht W2GMD <gba@orionlabs.io>
:copyright: Copyright 2016 Orion Labs, Inc.
:license: Apache License, Version 2.0
:source: <https://github.com/ampledata/rsuv3>
"""

from rsuv3.classes import RSUV3  # NOQA
from rsuv3.util import fix_frequency  # NOQA

__author__ = 'Greg Albrecht W2GMD <gba@orionlabs.io>'
__copyright__ = 'Copyright 2016 Orion Labs, Inc.'
__license__ = 'Apache License, Version 2.0'
